/*===========================================================================*/
/* SEED reader     |               parse_type52              |  station header */
/*===========================================================================*/
/*
	Name:		parse_type52
	Purpose:	parse a channel id (type 52) blockette into a globally-available
				table
	Usage:		void parse_type52 ();
				char *blockette;
				parse_type52 (blockette);
	Input:		blockette = pointer to beginning of blockette
	Output:		none
	Externals:	temp_char
				type52_head - 1st entry in table; defined in structures.h,
					allocated in globals.h
	Warnings:	none
	Errors:		none
	Called by:	process_stationh
	Calls to:	parse_int - get an integer from a blockette 
				parse_long - get a long integer from a blockette 
				parse_double - get a double from a blockette 
				parse_nchar - get a fixed-length string from a blockette 
				parse_varlstr - get a variable-length string from a blockette
	Algorithm:	parse the blockette into a globally-available table
	Notes:		none
	Problems:	none known
	References:	Halbert et al, 1988; see main routine
	Language:	C, hopefully ANSI standard
	Author:		Dennis O'Neill
	Revisions:	07/15/88  Dennis O'Neill  Initial preliminary release 0.9
				11/04/88  Dennis O'Neill  replaced "~" by STRINGTERM in
				                          parse_varlstr calls
				11/21/88  Dennis O'Neill  Production release 1.0
				08/19/89  Dennis O'Neill  streamlined linked list construction
				09/17/89  Dennis O'Neill  removed number_ch_comments
*/

#include "rdseed.h"

struct type52 *parse_type52 (blockette)
    char *blockette;								/* ptr to start of blockette */
{
	struct type52 *type52; 							/* genl member of table */
	char *blockette_ptr;						/* ptr to inside blockette */

	if ((type52 = (struct type52 *) malloc(sizeof(struct type52))) != NULL)
		bzero (type52, sizeof(struct type52));
	else return (type52);
	    
	/* point to beginning of information, past type and length */
	blockette_ptr = blockette;
	blockette_ptr += 7;

/*                 +=======================================+                 */
/*=================|   recover information from the type52   |=================*/
/*                 +=======================================+                 */

	/* recover location identifier */
	type52->location = parse_nchar (&blockette_ptr, 2);

	/* recover channel identifier */
	type52->channel = parse_nchar (&blockette_ptr, 3);

	/* recover subchannel identifier (for multiplexing) */
	type52->subchannel = parse_int (&blockette_ptr, 4);

	/* recover index for instrument ident in dictionary */
	type52->instrument_code = parse_int (&blockette_ptr, 3);

	/* recover optional instrument comment */
	if (*blockette_ptr != '~')
		type52->inst_comment = parse_varlstr (&blockette_ptr, STRINGTERM);
	else
	{
		type52->inst_comment = NULL;
		blockette_ptr += 1;
	}

	/* recover index for signal units in dictionary */
	type52->signal_units_code = parse_int (&blockette_ptr, 3);

	/* recover index for calibration units in dictionary */
	type52->calib_units_code = parse_int (&blockette_ptr, 3);

	/* recover latitude */
	type52->latitude = parse_double (&blockette_ptr, 10);

	/* recover longitude */
	type52->longitude = parse_double (&blockette_ptr, 11);

	/* recover elevation */
	type52->elevation = parse_double (&blockette_ptr, 7);

	/* recover local depth */
	type52->local_depth = parse_double (&blockette_ptr, 5);

	/* recover azimuth */
	type52->azimuth = parse_double (&blockette_ptr, 5);

	/* recover dip */
	type52->dip = parse_double (&blockette_ptr, 5);

	/* recover data format id lookup code */
	type52->format_code = parse_int (&blockette_ptr, 4);

	/* recover log2 of data record length */
	type52->log2drecl = parse_int (&blockette_ptr, 2);

	/* recover sample rate */
	type52->samplerate = parse_double (&blockette_ptr, 10);

	/* recover maximum clock drift tolerance */
	type52->clock_tolerance = parse_double (&blockette_ptr, 10);

	/* recover number of channel comment blockettes - old spec, reserved */
	/* type52->number_ch_comments = parse_int (&blockette_ptr, 4); */
	type52->reserved1 = parse_int (&blockette_ptr, 4);

	/* recover channel flags, if any */
	if (*blockette_ptr != '~')
		type52->channel_flag = parse_varlstr (&blockette_ptr, STRINGTERM);
	else blockette_ptr += 1;

	/* recover effective start date */
	type52->start = parse_varlstr (&blockette_ptr, STRINGTERM);

	/* recover effective end date, if any */
	if (*blockette_ptr != '~')
		type52->end = parse_varlstr (&blockette_ptr, STRINGTERM);
	else
	{
		blockette_ptr += 1;
		type52->end = NULL;
	}

	/* recover update flag */
	type52->update = parse_nchar (&blockette_ptr, 1);

	return (type52);
}

void free_type52 (type52)
    struct type52 *type52;
{
    if (type52 == NULL) return;
    if (type52->location)	free(type52->location);
    if (type52->channel)	free(type52->channel);
    if (type52->inst_comment)	free(type52->inst_comment);
    if (type52->channel_flag)	free(type52->channel_flag);
    if (type52->start)		free(type52->start);
    if (type52->end)		free(type52->end);
    if (type52->update)		free(type52->update);
    free (type52);
}
